package com.example.andersb.maps3;

import android.content.Context;
import android.location.Address;
import android.location.Criteria;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Build;
import android.support.v4.app.FragmentActivity;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.UiSettings;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class MapsActivity extends FragmentActivity implements LocationListener {

    private GoogleMap mMap; // Might be null if Google Play services APK is not available.
    private static final LatLng ROSKILDE = new LatLng(55.625622, 12.074981);
    private LocationManager locationManager;
    private String locationProvider;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maps);
        setUpMapIfNeeded();
        GetAddressFromNameTask task = new GetAddressFromNameTask(this);
        task.execute("Roskilde, Denmark"); // works, but sometimes really slow ...
        initializeLocationManager();
    }

    @Override
    protected void onResume() {
        Log.d("MAP", "onResume");
        super.onResume();
        setUpMapIfNeeded();

        final int minTime = 400;
        final int minDistance = 1;
        final LocationListener listener = this;
        locationManager.requestLocationUpdates(this.locationProvider, minTime, minDistance, listener);
    }

    @Override
    protected void onPause() {
        Log.d("MAP", "onPause");
        locationManager.removeUpdates(this);
    }

    /**
     * Sets up the map if it is possible to do so (i.e., the Google Play services APK is correctly
     * installed) and the map has not already been instantiated.. This will ensure that we only ever
     * call {@link #setUpMap()} once when {@link #mMap} is not null.
     * <p/>
     * If it isn't installed {@link SupportMapFragment} (and
     * {@link com.google.android.gms.maps.MapView MapView}) will show a prompt for the user to
     * install/update the Google Play services APK on their device.
     * <p/>
     * A user can return to this FragmentActivity after following the prompt and correctly
     * installing/updating/enabling the Google Play services. Since the FragmentActivity may not
     * have been completely destroyed during this process (it is likely that it would only be
     * stopped or paused), {@link #onCreate(Bundle)} may not be called again so we should call this
     * method in {@link #onResume()} to guarantee that it will be called.
     */
    private void setUpMapIfNeeded() {
        // Do a null check to confirm that we have not already instantiated the map.
        if (mMap == null) {
            // Try to obtain the map from the SupportMapFragment.
            mMap = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map))
                    .getMap();
            // Check if we were successful in obtaining the map.
            if (mMap != null) {
                setUpMap();
            } else {
                Toast.makeText(getBaseContext(), "Cannot find Google Maps", Toast.LENGTH_LONG).show();
                Log.e("MAP", "Cannot find Google Maps");
            }
        }
    }

    /**
     * This is where we can add markers or lines, add listeners or move the camera. In this case, we
     * just add a marker near Africa.
     * <p/>
     * This should only be called once and when we are sure that {@link #mMap} is not null.
     */
    private void setUpMap() {
        mMap.setMapType(GoogleMap.MAP_TYPE_HYBRID);
        mMap.setMyLocationEnabled(true);

        final UiSettings settings = mMap.getUiSettings();
        settings.setZoomControlsEnabled(true);
        settings.setMyLocationButtonEnabled(true);
        settings.setCompassEnabled(true);

        final BitmapDescriptor logo = BitmapDescriptorFactory.fromResource(R.drawable.ic_launcher);

        final MarkerOptions markerOptionsRoskilde = new MarkerOptions()
                .position(ROSKILDE).title("EASJ").snippet("It's here!")
                .icon(logo);
        final Marker roskilde = mMap.addMarker(markerOptionsRoskilde);

        //mMap.addMarker(new MarkerOptions().position(new LatLng(0, 0)).title("Marker"));
        //  mMap.addMarker(new MarkerOptions().position(ROSKILDE).title("Marker"));

        mMap.setOnMapClickListener(new GoogleMap.OnMapClickListener() {
            @Override
            public void onMapClick(LatLng latLng) {
             /*   final String message = "You clicked: " + latLng.latitude
                        + ", " + latLng.longitude;
                Toast.makeText(getBaseContext(), message, Toast.LENGTH_SHORT).show();*/
                final MarkerOptions options = new MarkerOptions().position(
                        latLng).title("biip");
                mMap.addMarker(options);

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.GINGERBREAD
                        && Geocoder.isPresent()) {

                    GetAddressFromLocationTask addressTask = new GetAddressFromLocationTask(
                            getBaseContext());

                    addressTask.execute(latLng);
                } else {
                    Toast.makeText(getBaseContext(),
                            "Geocoder not present", Toast.LENGTH_SHORT)
                            .show();
                }
            }
        });

        mMap.setOnMarkerClickListener(new GoogleMap.OnMarkerClickListener() {

            @Override
            public boolean onMarkerClick(final Marker marker) {
                if (marker.equals(roskilde)) {
                    Toast.makeText(getBaseContext(), "EASJ", Toast.LENGTH_SHORT).show();
                } else {
                    //    marker.remove();
                }
                return false;
            }
        });
    }


    /**
     * http://developer.android.com/training/location/display-address.html
     */
    private class GetAddressFromLocationTask extends AsyncTask<LatLng, Void, String> {
        private final Context mContext;

        public GetAddressFromLocationTask(final Context context) {
            this.mContext = context;
        }

        @Override
        protected String doInBackground(LatLng... locations) {
            Geocoder geocoder = new Geocoder(mContext);
            LatLng loc = locations[0];
            List<Address> addresses = null;
            try {
                final int maxResult = 1;
                // Reverse geo-coding: (lat,long) -> addresses
                addresses = geocoder.getFromLocation(loc.latitude, loc.longitude, maxResult);
            } catch (IOException e1) {
                Log.e("LocationSampleActivity",
                        "IO Exception in getFromLocation()");
                e1.printStackTrace();
                return ("IO Exception trying to get address");
            } catch (IllegalArgumentException e2) {
                // Error message to post in the log
                String errorString = "Illegal arguments "
                        + Double.toString(loc.latitude) + " , "
                        + Double.toString(loc.longitude)
                        + " passed to address service";
                Log.e("LocationSampleActivity", errorString);
                //e2.printStackTrace();
                return errorString;
            }
            // If the reverse geocode returned an address
            if (addresses != null && addresses.size() > 0) {
                // Get the first address
                Address address = addresses.get(0);
                /*
                 * Format the first line of address (if available), city, and
				 * country name.
				 */
                String addressText = String.format("%s, %s, %s",
                        // If there's a street address, add it
                        address.getMaxAddressLineIndex() > 0 ? address
                                .getAddressLine(0) : "",
                        // Locality is usually a city
                        address.getLocality(),
                        // The country of the address
                        address.getCountryName());
                // Return the text
                return addressText;
            } else {
                return "No address found";
            }
        }

        @Override
        protected void onPostExecute(final String result) {
            Toast.makeText(mContext, result, Toast.LENGTH_LONG).show();
        }
    }

    private class GetAddressFromNameTask extends AsyncTask<String, Void, List<Address>> {
        private final Context context;

        GetAddressFromNameTask(Context context) {
            this.context = context;
        }

        @Override
        protected List<Address> doInBackground(String... places) {
            if (!Geocoder.isPresent()) {
                Log.d("MAP", "Geocoder not present");
                return new ArrayList<Address>();
            }
            Geocoder geocoder = new Geocoder(context);
            int maxResults = 5;
            String place = places[0];
            try {
                // geo-coding: place -> address
                final List<Address> addresses = geocoder.getFromLocationName(place, maxResults);
                return addresses;
            } catch (IOException ex) {
                Toast.makeText(context, ex.toString(), Toast.LENGTH_LONG).show();
                Log.e("MAP", ex.toString());
                return new ArrayList<Address>();
            }
        }

        @Override
        protected void onPostExecute(List<Address> addresses) {
            super.onPostExecute(addresses);
            if (addresses.size() == 0) {
                Toast.makeText(context, "No address", Toast.LENGTH_SHORT).show();
                Log.d("MAP", "No address");
            }
            for (Address addr : addresses) {
                Toast.makeText(context, addr.getAddressLine(0), Toast.LENGTH_SHORT).show();
                Log.d("MAP", "Address: " + addr.getAddressLine(0));
            }
        }
    }

    //-------------------------------------------
    //	Summary: initialize location manager
    //-------------------------------------------
    private void initializeLocationManager() {
        //get the location manager
        this.locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);

        //define the location manager criteria
        Criteria criteria = new Criteria();
        this.locationProvider = locationManager.getBestProvider(criteria, false);
        Log.d("MAP", locationProvider.toString());
        Location location = locationManager.getLastKnownLocation(locationProvider);

        //initialize the location
        if(location != null) {
            onLocationChanged(location);
        }
    }

    //	Summary: Location Listener  methods
    // https://gist.github.com/ccabanero/6996756
    @Override
    public void onLocationChanged(Location location) {
        Log.d("MAP", "onLocationChanged");
        //when the location changes, update the map by zooming to the location
        final LatLng latLng = new LatLng(location.getLatitude(), location.getLongitude());
        CameraUpdate center = CameraUpdateFactory.newLatLng(latLng);
        //mMap.moveCamera(center);

        MarkerOptions marker = new MarkerOptions().position(latLng);
        mMap.addMarker(marker);

        //CameraUpdate zoom = CameraUpdateFactory.zoomTo(15);
        //this.mMap.animateCamera(zoom);
    }

    @Override
    public void onProviderDisabled(String arg0) {
        Log.d("MAP", "onProviderDisabled");
    }

    @Override
    public void onProviderEnabled(String arg0) {
        Log.d("MAP", "onProviderEnabled");
    }

    @Override
    public void onStatusChanged(String arg0, int arg1, Bundle arg2) {
        Log.d("MAP", "onStatusChanged");
    }
}
